(* Mathematica Package *)

(* Created by the Wolfram Workbench Oct 5, 2009 *)

BeginPackage["IntegrationTechniques`"]
(* Exported symbols added here with SymbolName::usage *) 

Unprotect[Int,RemoveInt,SIntegrate,AutoSimplify,PartsIntegrate]

Int::usage= "Int is a placeholder head for expressions used in displaying results from
PartsIntegrate and SubstituteIntegrate."

SIntegrate::usage = 
  "SIntegrate[fun,var,{sub-> u,subDx-> aDu}] gives the \
expression Int[a f[u],u] and when Int is replaced by Integrate will \
give a solution to Integrate[fun,var]."

AutoSimplify::usage = 
  "AutoSimplify is an option for SIntegrate that indicates \
whether the result should be automatically simplified. The default is \
False."

RemoveInt::usage = "RemoveInt[expr] converts any expression with head Int to an equivalent \
expression with the head Integrate and evaluates the result."

PartsIntegrate::usage = "PartsIntegrate[integrand,variable,{u,dv}] returns the result of \
integrating the integrand with respect to the variable using the \
method integral[u dv]= u v - integral[v du]";

Begin["`Private`"]

(*Definition for RemoveInt*)

RemoveInt := # /. Int :> Integrate &

(* Deifnitions for validTest a function to check arguments for SubstituteIntegrate *)

validTest[r1 : (a1_ -> b1_), r2 : (a2_ -> b2_), var_] := 
   Block[{der = D[a1, var]}, 
    TrueQ[Simplify[(DifferentialD[var] /. 
              Solve[der*DifferentialD[var] == DifferentialD[b1], 
                  DifferentialD[var]][[1, 1]]) == b2]]]
                  
(* Deifnitions for SubstituteIntegrate *)

Attributes[SIntegrate] = {HoldAll}

Options[SIntegrate] = {AutoSimplify -> False}

SIntegrate::incrct = "At least one of your substitutions `1` or `2` is incorrect";

SIntegrate[(epow : E^(var_))*(coef___)*(expr2 : 
     Cos[E^(var_)] | Sin[E^(var_)] | 
            Tan[E^(var_)] | Cot[E^(var_)] | Cosh[E^(var_)] | 
      Sinh[E^(var_)] | 
            Tanh[E^(var_)] | Coth[E^(var_)]), var_, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, 
     opts : OptionsPattern[]] /; validTest[r1, r2, var]:= 
 Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SIntegrate[(epow : E^((num_)?NumericQ*(var_Symbol)))*(coef___)*
       (expr2 : Cos[E^((num_)?NumericQ*(var_Symbol))] | 
            Sin[E^((num_)?NumericQ*(var_Symbol))] | 
            Tan[E^((num_)?NumericQ*(var_Symbol))] | Cot[E^(var_)] | 
            Cosh[E^((num_)?NumericQ*(var_Symbol))] | 
            Sinh[E^((num_)?NumericQ*(var_Symbol))] | 
            Tanh[E^((num_)?NumericQ*(var_Symbol))] | 
            Coth[E^((num_)?NumericQ*(var_Symbol))]), var_, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, 
     opts : OptionsPattern[]]/; validTest[r1, r2, var] := 
Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SIntegrate[(epow : E^(var_)^(num_))*(coef___)*(expr2 : 
     Cos[E^(var_)^(num_)] | 
            Sin[E^(var_)^(num_)] | Tan[E^(var_)^(num_)] | 
            Cot[E^(var_)^(num_)] | Cosh[E^(var_)^(num_)] | 
            Sinh[E^(var_)^(num_)] | Tanh[E^(var_)^(num_)] | 
            Coth[Power[(var_)^(num_)]]), 
  var_, {r1 : (exp_ -> u_Symbol), 
       r2 : (Dx_ -> expr__)}, opts : OptionsPattern[]]/; validTest[r1, r2, var] := 
   Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SIntegrate[(epow : E^((num2_)?NumericQ*(var_)^(num_)))*(coef___)*
       (expr2 : Cos[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Sin[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Tan[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Cot[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Cosh[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Sinh[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Tanh[E^((num2_)?NumericQ*(var_)^(num_))] | 
            Coth[E^((num2_)?NumericQ*(var_)^(num_))]), var_, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, 
     opts : OptionsPattern[]]/; validTest[r1, r2, var] := 
 Which[OptionValue[AutoSimplify]===False,
 Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u],
       OptionValue[AutoSimplify]===True,
       Simplify[ Int[epow*(coef*expr2*Dx /. {r1, r2}) /. 
       {DifferentialD[u] -> 1}, u]]
 ]

SIntegrate[integrnd:Sin[v_Symbol]/Cos[v_Symbol], 
     var_Symbol, {r1:(exp_ -> u_Symbol), r2:(Dx_ -> expr__)}, 
  opts:OptionsPattern[]]/; validTest[r1, r2, var] := Module[{newint},
 (*Print["def 1"];*)
  newint = DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
        DifferentialD[u], 4];
  Which[OptionValue[AutoSimplify] === False,
   Int[newint, u],
   OptionValue[AutoSimplify] === True,
   Simplify[Int[newint, u]]]
  ]

SIntegrate[
  integrnd:Cos[v_Symbol]/Sin[v_Symbol], 
     var_Symbol, {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 2"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Sin[v_Symbol]/
           Cos[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 3"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}],  DifferentialD[u], 4], u], 
  	OptionValue[AutoSimplify] === True, 
  	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}],  DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Cos[v_Symbol]/
           Sin[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
    ( (*Print["def 4"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u],
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Sin[v_Symbol]/
           Cos[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 5"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u], 
  	OptionValue[AutoSimplify] === True, 
  	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SIntegrate[
  integrnd:Sin[(num_)?NumericQ*(v_Symbol)]/
           Cos[(num_)?NumericQ*(v_Symbol)]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
     ((*Print["def 6"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u], 
  	OptionValue[AutoSimplify] === True, 
  	Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Cos[v_Symbol]/
           Sin[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
     ((*Print["def 7"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[
  integrnd:Cos[(num_)?NumericQ*(v_Symbol)]/
           Sin[(num_)?NumericQ*(v_Symbol)]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 8"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Sin[v_Symbol]/
           Cos[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
     ((*Print["def 9"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Sinh[v_Symbol]/
           Cosh[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 10"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u],
        OptionValue[AutoSimplify] === True, 
        Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[
  integrnd:Sinh[(num_)?NumericQ*(v_Symbol)]/
           Cosh[(num_)?NumericQ*(v_Symbol)]^(n_Integer), 
  var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 11"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u],
        OptionValue[AutoSimplify] === True, 
        Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[integrnd:Cosh[v_Symbol]/
           Sinh[v_Symbol]^(n_Integer), var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
     ((*Print["def 12"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[
  integrnd:Cosh[(num_)?NumericQ*(v_Symbol)]/
           Sinh[(num_)?NumericQ*(v_Symbol)]^(n_Integer), 
  var_Symbol, 
     {r1 : (exp_ -> u_Symbol), r2 : (Dx_ -> expr__)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
    ( (*Print["def 13"];*)
  Which[OptionValue[AutoSimplify] === False, 
  	Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u], 
       OptionValue[AutoSimplify] === True, 
       Simplify[Int[DeleteCases[ReleaseHold[HoldForm[integrnd*Dx] /. {r1, r2}], 
       DifferentialD[u], 4], u]]])

SIntegrate[integrnd_, var_Symbol, {r1 : (exp_ -> u_Symbol), 
       r2 : (Dx_DifferentialD -> expr_)}, opts:OptionsPattern[]]/; validTest[r1, r2, var] := 
      ( (*Print["def 14"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u], 
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u]]])

SIntegrate[integrnd_, var_Symbol, {r1 : (exp_ -> u_Symbol), 
       r2 : (Dx_DifferentialD -> Du_DifferentialD)}, opts:OptionsPattern[]] /; validTest[r1, r2, var]:= 
      ( (*Print["def 15"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[integrnd*Dx /. {r1, r2}, Du, 4], u], 
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[integrnd*Dx /. {r1, r2}, Du, 4], u]]])
   	
SIntegrate[integrnd_, var_Symbol, {r1 : (exp_ -> pow:Power[u_Symbol,n_Integer]), 
       r2 : (Dx_DifferentialD -> expr_)}, opts:OptionsPattern[]]/;D[pow,u]==Most[expr] := 
      ( (*Print["def 16"];*)
   Which[OptionValue[AutoSimplify] === False,
   	Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u], 
   	OptionValue[AutoSimplify] === True, 
   	Simplify[Int[DeleteCases[integrnd*Dx /. {r1, r2}, DifferentialD[u], 4], u]]])
   	
SIntegrate[integrnd_,var_Symbol,lis_List,OptionsPattern[]]/;Message[SIntegrate::incrct,lis[[1]],lis[[2]]]:=Null

(*Definitions for the function PartsIntegrate*)

PartsIntegrate::incrct="At least one of your choices `1` or `2` is incorrect."

PartsIntegrate[fun_Int, var_Symbol, {u_, dv_}] := 
 PartsIntegrate[fun[[1]], var, {u, dv}]

PartsIntegrate[fun_, var_Symbol, {u_, dv_}] := 
 Block[{du = D[u, var], v = Integrate[dv, var]}, 
  If[Not[Simplify[u dv === fun]], 
   Message[PartsIntegrate::incrct,u,dv];
   StringForm["PartsIntegrate[`1`,`2`,{`3`,`4`}]",fun,var,u,dv], 
   u v - Int[v du, var]]]

End[]

Protect[Int,RemoveInt,SIntegrate,AutoSimplify,PartsIntegrate]

EndPackage[]

