(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      8482,        253]
NotebookOptionsPosition[      7374,        210]
NotebookOutlinePosition[      7756,        227]
CellTagsIndexPosition[      7713,        224]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["The IntegrationTechniques Package", "Section",
 CellChangeTimes->{{3.464532346657558*^9, 3.4645323594009933`*^9}}],

Cell[CellGroupData[{

Cell["Description", "Subsection",
 CellChangeTimes->{{3.464532378105114*^9, 3.464532382057263*^9}}],

Cell["\<\
The IntegrationTechniques package contains functions to introduce the \
concepts of integration by parts and integration using substitution. After \
installing the package you can evaluate <<IntegrationTechniques` to load the \
package. Then evaluating ?PartsIntegrate or ?SIntegrate will give you links \
to the documentation for the package.\
\>", "Text",
 CellChangeTimes->{{3.464532386428913*^9, 3.464532603893504*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installing", "Subsection",
 CellChangeTimes->{{3.464532371635337*^9, 3.464532374057703*^9}}],

Cell["\<\
You can install the IntegrationTechniques package by clicking one of the \
buttons below. \
\>", "Text",
 CellChangeTimes->{{3.4645313439638968`*^9, 3.464531383002733*^9}}],

Cell[CellGroupData[{

Cell["Typically you would install to $BaseDirectory", "Item",
 CellChangeTimes->{{3.46453140965156*^9, 3.4645314231155376`*^9}}],

Cell["\<\
Installl to $UserBaseDirectory if you do not have permissions to write to the \
$BaseDirectory\
\>", "Item",
 CellChangeTimes->{{3.464531433404224*^9, 3.464531452020526*^9}, {
  3.464531856789316*^9, 3.464531872598174*^9}}],

Cell[TextData[{
 "Installing the package in both locations ",
 StyleBox["WILL ",
  FontWeight->"Bold"],
 " lead to problems"
}], "Item",
 CellChangeTimes->{{3.4645328828761377`*^9, 3.46453292304566*^9}}]
}, Open  ]],

Cell["\<\
On completion of the package install you file system will open to the \
directory where the package has been installed.\
\>", "Text",
 CellChangeTimes->{{3.4645322772654953`*^9, 3.464532331160501*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Button", "[", 
       RowBox[{"\"\<Install To $UserBaseDirectory\>\"", ",", 
        RowBox[{
         RowBox[{"CopyDirectory", "[", 
          RowBox[{
           RowBox[{"FileNameJoin", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"NotebookDirectory", "[", "]"}], ",", 
              "\"\<IntegrationTechniques\>\""}], "}"}], "]"}], ",", 
           RowBox[{"FileNameJoin", "[", 
            RowBox[{"{", 
             RowBox[{
             "$UserBaseDirectory", ",", "\"\<Applications\>\"", ",", 
              "\"\<IntegrationTechniques\>\""}], "}"}], "]"}]}], "]"}], ";;", 
         
         RowBox[{"SystemOpen", "[", 
          RowBox[{"FileNameJoin", "[", 
           RowBox[{"{", 
            RowBox[{"$UserBaseDirectory", ",", "\"\<Applications\>\""}], 
            "}"}], "]"}], "]"}]}], ",", 
        RowBox[{"Appearance", "\[Rule]", " ", "\"\<Palette\>\""}]}], "]"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"Button", "[", 
       RowBox[{"\"\<Install To $BaseDirectory\>\"", ",", 
        RowBox[{
         RowBox[{"CopyDirectory", "[", 
          RowBox[{
           RowBox[{"FileNameJoin", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"NotebookDirectory", "[", "]"}], ",", 
              "\"\<IntegrationTechniques\>\""}], "}"}], "]"}], ",", 
           RowBox[{"FileNameJoin", "[", 
            RowBox[{"{", 
             RowBox[{
             "$BaseDirectory", ",", "\"\<Applications\>\"", ",", 
              "\"\<IntegrationTechniques\>\""}], "}"}], "]"}]}], "]"}], ";", 
         RowBox[{"SystemOpen", "[", 
          RowBox[{"FileNameJoin", "[", 
           RowBox[{"{", 
            RowBox[{"$BaseDirectory", ",", "\"\<Applications\>\""}], "}"}], 
           "]"}], "]"}]}], ",", 
        RowBox[{"Appearance", "\[Rule]", " ", "\"\<Palette\>\""}]}], "]"}], 
      "}"}]}], "}"}], ",", 
   RowBox[{"Frame", "\[Rule]", " ", "True"}], ",", 
   RowBox[{"Background", "\[Rule]", " ", 
    RowBox[{"GrayLevel", "[", ".4", "]"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.464530555450591*^9, 3.464530667188884*^9}, {
   3.46453076439631*^9, 3.4645308803483553`*^9}, {3.464531244397101*^9, 
   3.4645312626246147`*^9}, {3.464531293183467*^9, 3.464531322651937*^9}, {
   3.464531914418046*^9, 3.4645319542925177`*^9}, {3.46453199186064*^9, 
   3.464532017837247*^9}, {3.464532052040018*^9, 3.464532053136758*^9}, 
   3.464532095353505*^9, {3.46453453559857*^9, 3.464534593409408*^9}, {
   3.464534997313642*^9, 3.464535008207481*^9}}],

Cell[BoxData[
 TagBox[GridBox[{
    {
     ButtonBox["\<\"Install To $UserBaseDirectory\"\>",
      Appearance->"Palette",
      ButtonFunction:>Span[
        CopyDirectory[
         FileNameJoin[{
           NotebookDirectory[], "IntegrationTechniques"}], 
         FileNameJoin[{$UserBaseDirectory, "Applications", 
           "IntegrationTechniques"}]], 
        SystemOpen[
         FileNameJoin[{$UserBaseDirectory, "Applications"}]]],
      Evaluator->Automatic,
      Method->"Preemptive"]},
    {
     ButtonBox["\<\"Install To $BaseDirectory\"\>",
      Appearance->"Palette",
      ButtonFunction:>(CopyDirectory[
         FileNameJoin[{
           NotebookDirectory[], "IntegrationTechniques"}], 
         FileNameJoin[{$BaseDirectory, "Applications", 
           "IntegrationTechniques"}]]; SystemOpen[
         FileNameJoin[{$BaseDirectory, "Applications"}]]),
      Evaluator->Automatic,
      Method->"Preemptive"]}
   },
   AutoDelete->False,
   GridBoxBackground->{"Columns" -> {{
        GrayLevel[0.4]}}},
   GridBoxFrame->{"ColumnsIndexed" -> {{{1, -1}, {1, -1}} -> True}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellChangeTimes->{3.464530672464437*^9, 3.464531287637088*^9, 
  3.464531324784726*^9, 3.4645321084464006`*^9, 3.464534599189909*^9, 
  3.464535017326868*^9}]
}, {2}]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using the Package", "Subsection",
 CellChangeTimes->{{3.464534479906191*^9, 3.46453448374116*^9}}],

Cell["\<\
To use the functions in the package you will first need to load the package.\
\>", "Text",
 CellChangeTimes->{{3.464534494031555*^9, 3.4645345245986223`*^9}}],

Cell["Loading the package", "MathCaption",
 CellChangeTimes->{{3.464532638897608*^9, 3.464532642502681*^9}}],

Cell[BoxData[
 RowBox[{"<<", "IntegrationTechniques`"}]], "Input",
 CellChangeTimes->{{3.464532645078279*^9, 3.464532653824379*^9}, {
  3.4645327880156937`*^9, 3.464532798035161*^9}, {3.4645328659506683`*^9, 
  3.46453286993428*^9}}],

Cell["Getting information for the PartsIntegrate function.", "MathCaption",
 CellChangeTimes->{{3.4645326756814423`*^9, 3.464532691000523*^9}}],

Cell[BoxData[
 RowBox[{"?", "PartsIntegrate"}]], "Input",
 CellChangeTimes->{{3.4645326945154448`*^9, 3.464532700920618*^9}}]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
WindowSize->{640, 750},
WindowMargins->{{281, Automatic}, {Automatic, 47}},
ShowSelection->True,
FrontEndVersion->"7.0 for Mac OS X x86 (32-bit) (February 18, 2009)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 120, 1, 67, "Section"],
Cell[CellGroupData[{
Cell[712, 27, 99, 1, 34, "Subsection"],
Cell[814, 30, 433, 7, 71, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1284, 42, 98, 1, 34, "Subsection"],
Cell[1385, 45, 182, 4, 26, "Text"],
Cell[CellGroupData[{
Cell[1592, 53, 128, 1, 20, "Item"],
Cell[1723, 56, 233, 5, 20, "Item"],
Cell[1959, 63, 203, 6, 20, "Item"]
}, Open  ]],
Cell[2177, 72, 211, 4, 41, "Text"],
Cell[CellGroupData[{
Cell[2413, 80, 2637, 61, 238, "Input"],
Cell[5053, 143, 1351, 35, 77, "Output"]
}, {2}]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6450, 184, 104, 1, 34, "Subsection"],
Cell[6557, 187, 168, 3, 26, "Text"],
Cell[6728, 192, 108, 1, 40, "MathCaption"],
Cell[6839, 195, 233, 4, 27, "Input"],
Cell[7075, 201, 143, 1, 40, "MathCaption"],
Cell[7221, 204, 125, 2, 27, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
